﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace SilverlightApplication1 {
	public class AnimationItemsControl : ItemsControl {
		private List<FrameworkElement> appendingItems = new List<FrameworkElement>();
		private Duration opacityDuration = new Duration(TimeSpan.FromSeconds(0.5));
		private Duration zoomDuration = new Duration(TimeSpan.FromSeconds(1));
		private double itemInterval = 0.5;

		protected override void OnItemsChanged(System.Collections.Specialized.NotifyCollectionChangedEventArgs e) {
			base.OnItemsChanged(e);

			if (e.Action == NotifyCollectionChangedAction.Reset) {
				appendingItems.Clear();
			} else if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add) {
				foreach (var item in e.NewItems) {
					this.UpdateLayout();
					var element = item as FrameworkElement;
					if (element == null) {
						element = base.ItemContainerGenerator.ContainerFromItem(item) as FrameworkElement;
					}
					if (element != null) {
						element.Opacity = 0.0;
						appendingItems.Add(element);
						Dispatcher.BeginInvoke(new Action(() => { AppendToList(); }));
					}
				}
			}
		}

		private void AppendToList() {
			for (var i = 0; i < appendingItems.Count; i++) {
				var item = appendingItems[i];
				AnimateItem(item, i);
			}
			appendingItems.Clear();
		}

		private void AnimateItem(FrameworkElement element, int delayCount) {
			if (!(element.RenderTransform is CompositeTransform)) {	// CompositeTransformがいい
				element.RenderTransform = new CompositeTransform() { ScaleX = 1, ScaleY = 1 };
			}

			var storyboard = new Storyboard();

			var opacityAnimation = new DoubleAnimation();
			opacityAnimation.From = 0.0;
			opacityAnimation.To = 1.0;
			opacityAnimation.Duration = opacityDuration;
			opacityAnimation.EasingFunction = new ExponentialEase() { EasingMode = EasingMode.EaseOut };
			Storyboard.SetTarget(opacityAnimation, element);
			Storyboard.SetTargetProperty(opacityAnimation, new PropertyPath(UIElement.OpacityProperty));
			element.Opacity = 0;
			storyboard.Children.Add(opacityAnimation);


			var zoomXAnimation = new DoubleAnimation();
			zoomXAnimation.From = 1.5;
			zoomXAnimation.To = 1;
			zoomXAnimation.Duration = zoomDuration;
			Storyboard.SetTarget(zoomXAnimation, element);
			Storyboard.SetTargetProperty(zoomXAnimation, new PropertyPath("(UIElement.RenderTransform).(CompositeTransform.ScaleX)"));
			storyboard.Children.Add(zoomXAnimation);

			var zoomYAnimation = new DoubleAnimation();
			zoomYAnimation.From = 1.5;
			zoomYAnimation.To = 1;
			zoomYAnimation.Duration = zoomDuration;
			Storyboard.SetTarget(zoomYAnimation, element);
			Storyboard.SetTargetProperty(zoomYAnimation, new PropertyPath("(UIElement.RenderTransform).(CompositeTransform.ScaleY)"));
			storyboard.Children.Add(zoomYAnimation);

			double delay = itemInterval * delayCount;
			storyboard.BeginTime = new TimeSpan?(TimeSpan.FromSeconds(delay));
			storyboard.Begin();
		}
	}
}
